@@ -117,7 +117,7 @@ opt_type(captcha_cmd) ->
 	fun(V) ->
 		V2 = misc:expand_keyword(<<"@SEMVER@">>, V,
 				    ejabberd_option:version()),
-		misc:expand_keyword(<<"@VERSION">>, V2,
+		misc:expand_keyword(<<"@VERSION@">>, V2,
 				    misc:semver_to_xxyy(ejabberd_option:version()))
 	end);
 opt_type(captcha_host) ->
@@ -264,6 +264,8 @@ opt_type(new_sql_schema) ->
     econf:bool();
 opt_type(update_sql_schema) ->
     econf:bool();
+opt_type(update_sql_schema_timeout) ->
+    econf:timeout(second, infinity);
 opt_type(oauth_access) ->
     econf:acl();
 opt_type(oauth_cache_life_time) ->
@@ -612,7 +614,8 @@ options() ->
      {negotiation_timeout, timer:seconds(120)},
      {net_ticktime, timer:seconds(60)},
      {new_sql_schema, ?USE_NEW_SQL_SCHEMA_DEFAULT},
-     {update_sql_schema, false},
+     {update_sql_schema, true},
+     {update_sql_schema_timeout, timer:minutes(5)},
      {oauth_access, none},
      {oauth_cache_life_time,
       fun(Host) -> ejabberd_config:get_option({cache_life_time, Host}) end},
