@@ -36,14 +36,15 @@
          install_contrib_modules/2,
          config_dir/0, get_commands_spec/0]).
 -export([modules_configs/0, module_ebin_dir/1]).
--export([compile_erlang_file/2, compile_elixir_file/2]).
--export([web_menu_node/3, web_page_node/5, get_page/3]).
+-export([compile_erlang_file/2, compile_elixir_files/2]).
+-export([web_menu_node/3, web_page_node/3, webadmin_node_contrib/3]).
 
 %% gen_server callbacks
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2,
 	 terminate/2, code_change/3]).
 
 -include("ejabberd_commands.hrl").
+-include("ejabberd_http.hrl").
 -include("ejabberd_web_admin.hrl").
 -include("logger.hrl").
 -include("translate.hrl").
@@ -593,17 +594,16 @@ compile_deps(LibDir) ->
 
 compile(LibDir) ->
     Bin = filename:join(LibDir, "ebin"),
-    Inc = filename:join(LibDir, "include"),
     Lib = filename:join(LibDir, "lib"),
     Src = filename:join(LibDir, "src"),
-    Options = [{outdir, Bin}, {i, Inc} | compile_options()],
+    Includes = [{i, Inc} || Inc <- filelib:wildcard(LibDir++"/../../**/include")],
+    Options = [{outdir, Bin}, {i, LibDir++"/.."} | Includes ++ compile_options()],
     filelib:ensure_dir(filename:join(Bin, ".")),
-    [copy(App, Bin) || App <- filelib:wildcard(Src++"/*.app")],
+    [copy(App, filename:join(Bin, filename:basename(App, ".src"))) || App <- filelib:wildcard(Src++"/*.app*")],
     compile_c_files(LibDir),
     Er = [compile_erlang_file(Bin, File, Options)
-          || File <- filelib:wildcard(Src++"/*.erl")],
-    Ex = [compile_elixir_file(Bin, File)
-          || File <- filelib:wildcard(Lib ++ "/**/*.ex")],
+          || File <- filelib:wildcard(Src++"/**/*.erl")],
+    Ex = compile_elixir_files(Bin, filelib:wildcard(Lib ++ "/**/*.ex")),
     compile_result(lists:flatten([Er, Ex])).
 
 compile_c_files(LibDir) ->
@@ -668,18 +668,34 @@ compile_erlang_file(Dest, File, ErlOptions) ->
     end.
 
 -ifdef(ELIXIR_ENABLED).
-compile_elixir_file(Dest, File) when is_list(Dest) and is_list(File) ->
-  compile_elixir_file(list_to_binary(Dest), list_to_binary(File));
+compile_elixir_files(_, []) ->
+    ok;
+compile_elixir_files(Dest, [File | _] = Files) when is_list(Dest) and is_list(File) ->
+  BinFiles = [list_to_binary(F) || F <- Files],
+  compile_elixir_files(list_to_binary(Dest), BinFiles);
 
-compile_elixir_file(Dest, File) ->
-  try 'Elixir.Kernel.ParallelCompiler':files_to_path([File], Dest, []) of
-    Modules when is_list(Modules) -> {ok, Modules}
+compile_elixir_files(Dest, Files) ->
+  try 'Elixir.Kernel.ParallelCompiler':compile_to_path(Files, Dest, [{return_diagnostics, true}]) of
+      {ok, Modules, []} when is_list(Modules) ->
+          {ok, Modules};
+      {ok, Modules, Warnings} when is_list(Modules) ->
+          ?WARNING_MSG("Warnings compiling module: ~n~p", [Warnings]),
+          {ok, Modules}
   catch
-    _ -> {error, {compilation_failed, File}}
+    A:B ->
+          ?ERROR_MSG("Problem ~p compiling Elixir files: ~p~nFiles: ~p", [A, B, Files]),
+          {error, {compilation_failed, Files}}
   end.
 -else.
-compile_elixir_file(_, File) ->
-    {error, {compilation_failed, File}}.
+compile_elixir_files(_, []) ->
+    ok;
+compile_elixir_files(_, Files) ->
+    ErrorString = "Attempted to compile Elixir files, but Elixir support is "
+        "not available in ejabberd. Try compiling ejabberd using "
+        "'./configure --enable-elixir' or './configure --with-rebar=mix'",
+    ?ERROR_MSG(ErrorString, []),
+    io:format("Error: " ++ ErrorString ++ "~n", []),
+    {error, {elixir_not_available, Files}}.
 -endif.
 
 install(Module, Spec, SrcDir, LibDir, Config) ->
@@ -748,8 +764,10 @@ fetch_rebar_deps(SrcDir) ->
             {ok, CurDir} = file:get_cwd(),
             file:set_cwd(SrcDir),
             filelib:ensure_dir(filename:join("deps", ".")),
-            lists:foreach(fun({_App, Cmd}) ->
-                        os:cmd("cd deps; "++Cmd++"; cd ..")
+            lists:foreach(fun({App, Cmd}) ->
+                        io:format("Fetching dependency ~s: ", [App]),
+                        Result = os:cmd("cd deps; "++Cmd++"; cd .."),
+                        io:format("~s", [Result])
                 end, Deps),
             file:set_cwd(CurDir)
     end.
@@ -763,6 +781,19 @@ rebar_deps(Script) ->
         _ ->
             []
     end.
+
+rebar_dep({App, Version, Git}) when Version /= ".*" ->
+    AppS = atom_to_list(App),
+    Help = os:cmd("mix hex.package"),
+    case string:find(Help, "mix hex.package fetch") /= nomatch of
+        true ->
+            {App, "mix hex.package fetch "++AppS++" "++Version++" --unpack"};
+        false ->
+            io:format("I'll download ~p using git because I can't use Mix "
+                      "to fetch from hex.pm:~n~s", [AppS, Help]),
+            rebar_dep({App, ".*", Git})
+    end;
+
 rebar_dep({App, _, {git, Url}}) ->
     {App, "git clone "++Url++" "++filename:basename(App)};
 rebar_dep({App, _, {git, Url, {branch, Ref}}}) ->
@@ -884,21 +915,21 @@ get_commit_details2(Path) ->
     end.
 
 parse_details(Body) ->
-    {Contents} = jiffy:decode(Body),
+    Contents = misc:json_decode(Body),
 
-    {_, {Commit}} = lists:keyfind(<<"commit">>, 1, Contents),
-    {_, Sha} = lists:keyfind(<<"sha">>, 1, Commit),
-    {_, CommitHtmlUrl} = lists:keyfind(<<"html_url">>, 1, Commit),
+    {ok, Commit} = maps:find(<<"commit">>, Contents),
+    {ok, Sha} = maps:find(<<"sha">>, Commit),
+    {ok, CommitHtmlUrl} = maps:find(<<"html_url">>, Commit),
 
-    {_, {Commit2}} = lists:keyfind(<<"commit">>, 1, Commit),
-    {_, Message} = lists:keyfind(<<"message">>, 1, Commit2),
-    {_, {Author}} = lists:keyfind(<<"author">>, 1, Commit2),
-    {_, AuthorName} = lists:keyfind(<<"name">>, 1, Author),
-    {_, {Committer}} = lists:keyfind(<<"committer">>, 1, Commit2),
-    {_, Date} = lists:keyfind(<<"date">>, 1, Committer),
+    {ok, Commit2} = maps:find(<<"commit">>, Commit),
+    {ok, Message} = maps:find(<<"message">>, Commit2),
+    {ok, Author} = maps:find(<<"author">>, Commit2),
+    {ok, AuthorName} = maps:find(<<"name">>, Author),
+    {ok, Committer} = maps:find(<<"committer">>, Commit2),
+    {ok, Date} = maps:find(<<"date">>, Committer),
 
-    {_, {Links}} = lists:keyfind(<<"_links">>, 1, Contents),
-    {_, Html} = lists:keyfind(<<"html">>, 1, Links),
+    {ok, Links} = maps:find(<<"_links">>, Contents),
+    {ok, Html} = maps:find(<<"html">>, Links),
 
     #{sha => Sha,
       date => Date,
@@ -924,26 +955,156 @@ parse_details(Body) ->
            )
        ).
 
-web_menu_node(Acc, _Node, Lang) ->
-    Acc ++ [{<<"contrib">>, translate:translate(Lang, ?T("Contrib Modules"))}].
+%% @format-begin
 
-web_page_node(_, Node, [<<"contrib">>], Query, Lang) ->
-    Res = rpc:call(Node, ?MODULE, get_page, [Node, Query, Lang]),
-    {stop, Res};
-web_page_node(Acc, _, _, _, _) ->
+web_menu_node(Acc, _Node, _Lang) ->
+    Acc
+    ++ [{<<"contrib">>, <<"Contrib Modules (Detailed)">>},
+        {<<"contrib-api">>, <<"Contrib Modules (API)">>}].
+
+web_page_node(_,
+              Node,
+              #request{path = [<<"contrib">>],
+                       q = Query,
+                       lang = Lang} =
+                  R) ->
+    Title =
+        ?H1GL(<<"Contrib Modules (Detailed)">>,
+              <<"../../developer/extending-ejabberd/modules/#ejabberd-contrib">>,
+              <<"ejabberd-contrib">>),
+    Res = [ejabberd_cluster:call(Node,
+                                 ejabberd_web_admin,
+                                 make_command,
+                                 [webadmin_node_contrib,
+                                  R,
+                                  [{<<"node">>, Node}, {<<"query">>, Query}, {<<"lang">>, Lang}],
+                                  []])],
+    {stop, Title ++ Res};
+web_page_node(_, Node, #request{path = [<<"contrib-api">> | RPath]} = R) ->
+    Title =
+        ?H1GL(<<"Contrib Modules (API)">>,
+              <<"../../developer/extending-ejabberd/modules/#ejabberd-contrib">>,
+              <<"ejabberd-contrib">>),
+    _TableInstalled = make_table_installed(Node, R, RPath),
+    _TableAvailable = make_table_available(Node, R, RPath),
+    TableInstalled = make_table_installed(Node, R, RPath),
+    TableAvailable = make_table_available(Node, R, RPath),
+    Res = [?X(<<"hr">>),
+           ?XAC(<<"h2">>, [{<<"id">>, <<"specs">>}], <<"Specs">>),
+           ?XE(<<"blockquote">>,
+               [ejabberd_cluster:call(Node,
+                                      ejabberd_web_admin,
+                                      make_command,
+                                      [modules_update_specs, R])]),
+           ?X(<<"hr">>),
+           ?XAC(<<"h2">>, [{<<"id">>, <<"installed">>}], <<"Installed">>),
+           ?XE(<<"blockquote">>,
+               [ejabberd_cluster:call(Node,
+                                      ejabberd_web_admin,
+                                      make_command,
+                                      [modules_installed, R, [], [{only, presentation}]]),
+                ejabberd_cluster:call(Node,
+                                      ejabberd_web_admin,
+                                      make_command,
+                                      [module_uninstall, R, [], [{only, presentation}]]),
+                ejabberd_cluster:call(Node,
+                                      ejabberd_web_admin,
+                                      make_command,
+                                      [module_upgrade, R, [], [{only, presentation}]]),
+                TableInstalled]),
+           ?X(<<"hr">>),
+           ?XAC(<<"h2">>, [{<<"id">>, <<"available">>}], <<"Available">>),
+           ?XE(<<"blockquote">>,
+               [ejabberd_cluster:call(Node,
+                                      ejabberd_web_admin,
+                                      make_command,
+                                      [modules_available, R, [], [{only, presentation}]]),
+                ejabberd_cluster:call(Node,
+                                      ejabberd_web_admin,
+                                      make_command,
+                                      [module_install, R, [], [{only, presentation}]]),
+                TableAvailable,
+                ejabberd_cluster:call(Node, ejabberd_web_admin, make_command, [module_check, R])])],
+    {stop, Title ++ Res};
+web_page_node(Acc, _, _) ->
     Acc.
 
-get_page(Node, Query, Lang) ->
+make_table_installed(Node, R, RPath) ->
+    Columns = [<<"Name">>, <<"Summary">>, <<"">>, <<"">>],
+    ModulesInstalled =
+        ejabberd_cluster:call(Node,
+                              ejabberd_web_admin,
+                              make_command_raw_value,
+                              [modules_installed, R, []]),
+    Rows =
+        lists:map(fun({Name, Summary}) ->
+                     NameBin = misc:atom_to_binary(Name),
+                     Upgrade =
+                         ejabberd_cluster:call(Node,
+                                               ejabberd_web_admin,
+                                               make_command,
+                                               [module_upgrade,
+                                                R,
+                                                [{<<"module">>, NameBin}],
+                                                [{only, button}, {input_name_append, [NameBin]}]]),
+                     Uninstall =
+                         ejabberd_cluster:call(Node,
+                                               ejabberd_web_admin,
+                                               make_command,
+                                               [module_uninstall,
+                                                R,
+                                                [{<<"module">>, NameBin}],
+                                                [{only, button},
+                                                 {style, danger},
+                                                 {input_name_append, [NameBin]}]]),
+                     {?C(NameBin), ?C(list_to_binary(Summary)), Upgrade, Uninstall}
+                  end,
+                  ModulesInstalled),
+    ejabberd_web_admin:make_table(200, RPath, Columns, Rows).
+
+make_table_available(Node, R, RPath) ->
+    Columns = [<<"Name">>, <<"Summary">>, <<"">>],
+    ModulesAll =
+        ejabberd_cluster:call(Node,
+                              ejabberd_web_admin,
+                              make_command_raw_value,
+                              [modules_available, R, []]),
+    ModulesInstalled =
+        ejabberd_cluster:call(Node,
+                              ejabberd_web_admin,
+                              make_command_raw_value,
+                              [modules_installed, R, []]),
+    ModulesNotInstalled =
+        lists:filter(fun({Mod, _}) -> not lists:keymember(Mod, 1, ModulesInstalled) end,
+                     ModulesAll),
+    Rows =
+        lists:map(fun({Name, Summary}) ->
+                     NameBin = misc:atom_to_binary(Name),
+                     Install =
+                         ejabberd_cluster:call(Node,
+                                               ejabberd_web_admin,
+                                               make_command,
+                                               [module_install,
+                                                R,
+                                                [{<<"module">>, NameBin}],
+                                                [{only, button}, {input_name_append, [NameBin]}]]),
+                     {?C(NameBin), ?C(list_to_binary(Summary)), Install}
+                  end,
+                  ModulesNotInstalled),
+    ejabberd_web_admin:make_table(200, RPath, Columns, Rows).
+
+webadmin_node_contrib(Node, Query, Lang) ->
     QueryRes = list_modules_parse_query(Query),
-    Title = ?H1GL(translate:translate(Lang, ?T("Contrib Modules")),
-                  <<"../../developer/extending-ejabberd/modules/#ejabberd-contrib">>,
-                  <<"ejabberd-contrib">>),
     Contents = get_content(Node, Query, Lang),
-    Result = case QueryRes of
-                 ok -> [?XREST(?T("Submitted"))];
-                 nothing -> []
-             end,
-    Title ++ Result ++ Contents.
+    Result =
+        case QueryRes of
+            ok ->
+                [?XREST(?T("Submitted"))];
+            nothing ->
+                []
+        end,
+    Result ++ Contents.
+%% @format-end
 
 get_module_home(Module, Attrs) ->
     case get_module_information(home, Attrs) of
@@ -1064,7 +1225,7 @@ is_elixir_module(Module) ->
           filelib:wildcard(Src++"/*.{erl}")} of
         {[_ | _], []} ->
             true;
-        {[], [_ | _]} ->
+        {[], _} ->
             false
     end.
 
