@@ -197,11 +197,8 @@ perform_call(Command, Args, Req, Version) ->
 %% Be tolerant to make API more easily usable from command-line pipe.
 extract_args(<<"\n">>) -> [];
 extract_args(Data) ->
-    case jiffy:decode(Data) of
-        List when is_list(List) -> List;
-        {List} when is_list(List) -> List;
-        Other -> [Other]
-    end.
+    Maps = misc:json_decode(Data),
+    maps:to_list(Maps).
 
 % get API version N from last "vN" element in URL path
 get_api_version(#request{path = Path}) ->
@@ -335,21 +332,40 @@ format_arg({Elements},
 		 ({Val}) when is_list(Val) ->
 		      format_arg({Val}, Tuple)
 	      end, Elements);
+format_arg(Map,
+	   {list, {_ElementDefName, {tuple, [{_Tuple1N, Tuple1S}, {_Tuple2N, Tuple2S}]}}})
+    when is_map(Map) andalso
+	 (Tuple1S == binary orelse Tuple1S == string) ->
+    maps:fold(
+	fun(K, V, Acc) ->
+	    [{format_arg(K, Tuple1S), format_arg(V, Tuple2S)} | Acc]
+	end, [], Map);
 format_arg(Elements,
 	   {list, {_ElementDefName, {list, _} = ElementDefFormat}})
     when is_list(Elements) ->
     [{format_arg(Element, ElementDefFormat)}
      || Element <- Elements];
+
+%% Covered by command_test_list and command_test_list_tuple
 format_arg(Elements,
 	   {list, {_ElementDefName, ElementDefFormat}})
     when is_list(Elements) ->
     [format_arg(Element, ElementDefFormat)
      || Element <- Elements];
+
 format_arg({[{Name, Value}]},
 	   {tuple, [{_Tuple1N, Tuple1S}, {_Tuple2N, Tuple2S}]})
   when Tuple1S == binary;
        Tuple1S == string ->
     {format_arg(Name, Tuple1S), format_arg(Value, Tuple2S)};
+
+%% Covered by command_test_tuple and command_test_list_tuple
+format_arg(Elements,
+	   {tuple, ElementsDef})
+  when is_map(Elements) ->
+    list_to_tuple([element(2, maps:find(atom_to_binary(Name, latin1), Elements))
+                   || {Name, _Format} <- ElementsDef]);
+
 format_arg({Elements},
 	   {tuple, ElementsDef})
     when is_list(Elements) ->
@@ -366,10 +382,12 @@ format_arg({Elements},
 			  end
 		  end, ElementsDef),
     list_to_tuple(F);
+
 format_arg(Elements, {list, ElementsDef})
     when is_list(Elements) and is_atom(ElementsDef) ->
     [format_arg(Element, ElementsDef)
      || Element <- Elements];
+
 format_arg(Arg, integer) when is_integer(Arg) -> Arg;
 format_arg(Arg, binary) when is_list(Arg) -> process_unicode_codepoints(Arg);
 format_arg(Arg, binary) when is_binary(Arg) -> Arg;
@@ -455,6 +473,7 @@ format_result(Els, {Name, {list, {_, {tuple, [{_, atom}, _]}} = Fmt}}) ->
 format_result(Els, {Name, {list, {_, {tuple, [{name, string}, {value, _}]}} = Fmt}}) ->
     {misc:atom_to_binary(Name), {[format_result(El, Fmt) || El <- Els]}};
 
+%% Covered by command_test_list and command_test_list_tuple
 format_result(Els, {Name, {list, Def}}) ->
     {misc:atom_to_binary(Name), [element(2, format_result(El, Def)) || El <- Els]};
 
@@ -468,9 +487,11 @@ format_result(Tuple, {_Name, {tuple, [{name, string}, {value, _} = ValFmt]}}) ->
     {_, Val2} = format_result(Val, ValFmt),
     {iolist_to_binary(Name2), Val2};
 
+%% Covered by command_test_tuple and command_test_list_tuple
 format_result(Tuple, {Name, {tuple, Def}}) ->
     Els = lists:zip(tuple_to_list(Tuple), Def),
-    {misc:atom_to_binary(Name), {[format_result(El, ElDef) || {El, ElDef} <- Els]}};
+    Els2 = [format_result(El, ElDef) || {El, ElDef} <- Els],
+    {misc:atom_to_binary(Name), maps:from_list(Els2)};
 
 format_result(404, {_Name, _}) ->
     "not_found".
@@ -495,10 +516,10 @@ invalid_token_response() ->
 badrequest_response() ->
     badrequest_response(<<"400 Bad Request">>).
 badrequest_response(Body) ->
-    json_response(400, jiffy:encode(Body)).
+    json_response(400, misc:json_encode(Body)).
 
 json_format({Code, Result}) ->
-    json_response(Code, jiffy:encode(Result));
+    json_response(Code, misc:json_encode(Result));
 json_format({HTMLCode, JSONErrorCode, Message}) ->
     json_error(HTMLCode, JSONErrorCode, Message).
 
@@ -509,9 +530,9 @@ json_response(Code, Body) when is_integer(Code) ->
 %% message is binary
 json_error(HTTPCode, JSONCode, Message) ->
     {HTTPCode, ?HEADER(?CT_JSON),
-     jiffy:encode({[{<<"status">>, <<"error">>},
-                    {<<"code">>, JSONCode},
-                    {<<"message">>, Message}]})
+     misc:json_encode(#{<<"status">> => <<"error">>,
+                    <<"code">> =>  JSONCode,
+                    <<"message">> => Message})
     }.
 
 log(Call, Args, {Addr, Port}) ->
@@ -534,11 +555,11 @@ mod_options(_) ->
 mod_doc() ->
     #{desc =>
 	  [?T("This module provides a ReST interface to call "
-              "https://docs.ejabberd.im/developer/ejabberd-api[ejabberd API] "
+              "_`../../developer/ejabberd-api/index.md|ejabberd API`_ "
 	      "commands using JSON data."), "",
 	   ?T("To use this module, in addition to adding it to the 'modules' "
 	      "section, you must also enable it in 'listen' -> 'ejabberd_http' -> "
-              "http://../listen-options/#request-handlers[request_handlers]."), "",
+              "_`listen-options.md#request_handlers|request_handlers`_."), "",
 	   ?T("To use a specific API version N, when defining the URL path "
 	      "in the request_handlers, add a 'vN'. "
 	      "For example: '/api/v2: mod_http_api'"), "",
