@@ -28,7 +28,7 @@
 -protocol({xep, 313, '0.6.1', '15.06', "", ""}).
 -protocol({xep, 334, '0.2'}).
 -protocol({xep, 359, '0.5.0'}).
--protocol({xep, 425, '0.2.1', '23.04', "", ""}).
+-protocol({xep, 425, '0.3.0', '24.06', "", ""}).
 -protocol({xep, 441, '0.2.0'}).
 
 -behaviour(gen_mod).
@@ -1598,7 +1598,9 @@ mod_doc() ->
     #{desc =>
           ?T("This module implements "
              "https://xmpp.org/extensions/xep-0313.html"
-             "[XEP-0313: Message Archive Management]. "
+             "[XEP-0313: Message Archive Management] and "
+             "https://xmpp.org/extensions/xep-0441.html"
+             "[XEP-0441: Message Archive Management Preferences]. "
              "Compatible XMPP clients can use it to store their "
              "chat history on the server."),
       opts =>
@@ -1676,7 +1678,7 @@ mod_doc() ->
             #{value => "true | false",
               desc =>
                   ?T("When this option is disabled, for each individual "
-		     "subscriber a separa mucsub message is stored. With this "
+		     "subscriber a separate mucsub message is stored. With this "
 		     "option enabled, when a user fetches archive virtual "
 		     "mucsub, messages are generated from muc archives. "
 		     "The default value is 'false'.")}}]}.
