@@ -412,6 +412,12 @@ send_welcome_message(JID) ->
     case mod_register_opt:welcome_message(Host) of
       {<<"">>, <<"">>} -> ok;
       {Subj, Body} ->
+	  ejabberd_router:route(
+	    #message{from = jid:make(Host),
+		     to = JID,
+		     type = chat,
+		     subject = xmpp:mk_text(Subj),
+		     body = xmpp:mk_text(<<Subj/binary, "\n\n", Body/binary>>)}),
 	  ejabberd_router:route(
 	    #message{from = jid:make(Host),
 		     to = JID,
@@ -671,7 +677,7 @@ mod_doc() ->
            {captcha_protected,
             #{value => "true | false",
               desc =>
-                  ?T("Protect registrations with http://../basic/#captcha[CAPTCHA]. "
+                  ?T("Protect registrations with _`basic.md#captcha|CAPTCHA`_. "
                      "The default is 'false'.")}},
            {ip_access,
             #{value => ?T("AccessName"),
@@ -702,4 +708,13 @@ mod_doc() ->
             #{value => "{subject: Subject, body: Body}",
               desc =>
                   ?T("Set a welcome message that is sent to each newly registered account. "
-                     "The message will have subject 'Subject' and text 'Body'.")}}]}.
+                     "The message will have subject 'Subject' and text 'Body'."),
+              example =>
+                    ["modules:",
+                     "  mod_register:",
+                     "    welcome_message:",
+                     "      subject: \"Welcome!\"",
+                     "      body: |-",
+                     "        Hi!",
+                     "        Welcome to this XMPP server"]}}
+          ]}.
