@@ -667,18 +667,21 @@ mod_doc() ->
                      "of vCard. Since the representation has no attributes, "
                      "the mapping is straightforward."),
               example =>
-                  [{?T("For example, the following XML representation of vCard:"),
-                    ["<vCard xmlns='vcard-temp'>",
-                     "  <FN>Conferences</FN>",
-                     "  <ADR>",
-                     "    <WORK/>",
-                     "    <STREET>Elm Street</STREET>",
-                     "  </ADR>",
-                     "</vCard>"]},
-                   {?T("will be translated to:"),
-                    ["vcard:",
-                     "  fn: Conferences",
-                     "  adr:",
-                     "    -",
-                     "      work: true",
-                     "      street: Elm Street"]}]}}]}.
+                  ["# This XML representation of vCard:",
+                   "# ",
+                   "#   <vCard xmlns='vcard-temp'>",
+                   "#     <FN>Conferences</FN>",
+                   "#     <ADR>",
+                   "#       <WORK/>",
+                   "#       <STREET>Elm Street</STREET>",
+                   "#     </ADR>",
+                   "#   </vCard>",
+                   "# ",
+                   "# is translated to:",
+                   "# ",
+                   "vcard:",
+                   "  fn: Conferences",
+                   "  adr:",
+                   "    -",
+                   "      work: true",
+                   "      street: Elm Street"]}}]}.
