@@ -7,7 +7,7 @@ check_xep()
     [ -f $BASE/doc/$xep ] || curl -s -o $BASE/doc/$xep https://xmpp.org/extensions/$xep.html
     title=$(sed '/<title>/!d;s/.*<title>\(.*\)<\/title>.*/\1/' $BASE/doc/$xep)
     vsn=$(grep -A1 Version $BASE/doc/$xep | sed '/<dd>/!d;q' | sed 's/.*>\(.*\)<.*/\1/')
-    imp=$(grep "{xep, $int," $BASE/src/* | sed "s/.*src\/\(.*\).erl.*'\([0-9.-]*\)'.*/\1 \2/")
+    imp=$(grep "{xep, $int," $BASE/src/* | sed "s/.*src\/\(.*\).erl.*[0-9], '\([0-9.-]*\)'.*/\1 \2/")
     [ "$imp" == "" ] && imp="NA 0.0"
     echo "$title;$vsn;${imp/ /;}"
 }
